/**********************************************************  SigFPGA  *********************************************************

	Red Rapids
	797 North Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	The following code demonstrates the operation of all Red Rapids SigFPGA PCI Express products.

	Note:	The application can be applied to the maximum number of Red Rapids devices (MAX_NUM_DEVICES) supported, but only
			a single device (DEV0) is demonstrated.

	Rev History:  07-14-2015:	Changed call to the modified MemTest() function.
				  12-08-2014:	Added Model 371 support.
				  08-20-2014:	Added SRAM test and clarified error reporting.
				  07-21-2014:	Added Model 377 support and restructured default setting files.
				  05-21-2014:	Added power monitor and SRAM test functions.
				  01-10-2014:	Inital release.

	Copyright 2014, Red Rapids, Inc.

**********************************************************  SigFPGA  *********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "adapter.h"					// Adpater device driver common to all Red Rapids PCI Express products.
#include "sigfpga.h"					// SigFPGA application.
#include "dma_functions.h"				// Direct Memory Access (DMA) functions.
#include "common_functions.h"			// Functions common to all Signal Stream hardware and channel types.
#include "channel_functions.h"			// Functions that apply to individual Signal Stream channels.
#include "device_functions.h"			// Functions that apply to specific devices on the Signal Stream hardware.
#include "product_functions.h"			// Functions that apply to specific Signal Stream products.
#include "utility_functions.h"			// Functions specific to this application example.
#include "default_hardware_config.h"	// Default hardware configuration values.
#include "default_operating_config.h"	// Default operating configuration values.

int main()
{
	s_Adapter *p_HW[MAX_NUM_DEVICES];			// Hardware handle to the Red Rapids device.
	s_User User;								// User defined structure passed to the interrupt service routine (ISR).
	unsigned long Status = 0;					// Variable to store status returned by function calls.
	unsigned long DMABuffer[8];					// Array to store DMA buffer allocation status for upt to eight channels (success = 0, failure = 1).
	unsigned long Performance[4] = {0,0,0,0};	// Array to store PCI RX throughput (0), TX throughput (1), RX latency (2) and TX latency (3) measurements.
	unsigned long DataBuffer[8][1024];			// Array to store data buffer allocation status for up to eight channels (success = 0, failure = 1).
	unsigned long Handle[MAX_NUM_DEVICES];		// Array to store hardware handle allocation status for each device (success = 0, failure = 1).
	unsigned long devcnt = 0;					// Variable to index device numbers in loops.
	unsigned long chancnt = 0;					// Variable to index channel numbers in loops.
	size_t pagecnt = 0;							// Variable to index page numbers in loops.
	size_t index = 0;							// Variable to index array elements in loops.
	s_Channel RXChannel[8];						// Structure to store up to eight channels of receiver configuration variables.
	s_Channel TXChannel[8];						// Structure to store up to eight channels of transmitter configuration variables.
	s_ErrorStatus ErrorStatus;					// Structure to store the results returned by a fault status query to the device.
	s_Temperature Temperature;					// Structure to store the results returned by the temperature monitor on the device.
	s_Power Power;								// Structure to store the results returned by the power monitor on the device.
	double FsDivisor = 8.0;						// Variable to set the transmitter output frequency for demonstration (Freq = Fs / FsDivisor).
	char datafile[8][12] = {"rx1data.txt", "rx2data.txt", "rx3data.txt", "rx4data.txt", "rx5data.txt", "rx6data.txt", "rx7data.txt", "rx8data.txt"};	// Files to store receiver data.
	char dmafile[8][12] = {"dma1buf.txt", "dma2buf.txt", "dma3buf.txt", "dma4buf.txt", "dma5buf.txt", "dma6buf.txt", "dma7buf.txt", "dma8buf.txt"};		// Files to store receiver dma buffer contents.

	printf("******************************** SigFPGA ********************************\n");

	//////////////////////////////////////////////  Initialize Hardware Handle  //////////////////////////////////////////////

	printf("\nInitializing structures...\n");

	// Check the DMA configuration settings for possible errors.
	for (index = 1; index <= 4; index++)
		if(DMAConfigCheck(RXDefault.DMABuffer, index) || DMAConfigCheck(TXDefault.DMABuffer, index))
		{
			// This is a fatal error since the DMA buffers cannot be correctly configured.
			switch (index)
			{
				case 1: {printf("\tERROR: Illegal channel DMA buffer BurstSizeB configuration setting.\n"); break;}
				case 2: {printf("\tERROR: Illegal channel DMA buffer BurstCount configuration setting.\n"); break;}
				case 3: {printf("\tERROR: Illegal channel DMA buffer PageCount configuration setting.\n"); break;}
				case 4: {printf("\tERROR: Illegal channel DMA buffer PagesPerMark configuration setting.\n"); break;}
			}
			Status++;
		}

	// Initialize all eight receiver and transmitter channel configuration structures even though they may not all be assigned to the hardware.
	for (chancnt = 1; chancnt <= 8; chancnt++)
	{
		// Initialize all four receiver channel configuration structures to the same default value in this example.
		RXChannel[chancnt-1] = RXDefault;
		// Initialize all four transmitter channel configuration structures to the same default value in this example.
		TXChannel[chancnt-1] = TXDefault;
	}

	// Initialize the hardware handle for each device.
	for (devcnt = 0; devcnt < MAX_NUM_DEVICES; devcnt++)
	{
		// Allocate memory for the structure if there were no prior errors.
		if (!Status)
		{
			p_HW[devcnt] = (s_Adapter*) malloc(sizeof(s_Adapter));
			if (p_HW[devcnt] == NULL)
			{
				// This is a fatal error because there is no hardware handle available.
				printf("\tERROR: Unable to allocate memory for the device #%lu hardware handle.\n", devcnt);
				Status++;
				// Mark this handle as a failed allocation.
				Handle[devcnt] = 1;
			}
			else
			{
				// Mark this handle as a successful allocation.
				Handle[devcnt] = 0;
				// Initialize each member of the structure.
				Adapter_Zero(p_HW[devcnt]);
				// Set a unique device number for each hardware handle.
				p_HW[devcnt]->DevNum = devcnt;
				// Set the user defined pointer to pass information to the ISR in this application.
				p_HW[devcnt]->User = &User;
			}
		}
		// Mark the handle as a failed memory allocation if there were prior errors.
		else
			Handle[devcnt] = 1;
	}

	//////////////////////////////////////////////////  Configure Hardware  //////////////////////////////////////////////////
	if (!Status)
	{
		printf("\nOpening device #%u on the host...\n", DEV0);

		// Open the Red Rapids device on the bus and only continue if successful.
		if (!BusOpen(p_HW[DEV0]))
		{
			// Print the firmware revision date to the screen.
			printf("\tFirmware revision (MMDDYYYY): %08lX\n", FirmwareRevision(p_HW[DEV0]));

			// Software reset the device.
			Status += SoftwareReset(p_HW[DEV0]);

			// Peform model number specific hardware configuration procedures.
			if (!Status)
				switch (ModelNumber(p_HW[DEV0]))
				{
					case 0x371:	// Model 371
					{
						printf("\nInitializing Red Rapids Model 371...\n");
						// Initialize hardware specific to the Model 371.
						Status += M371Initialize(p_HW[DEV0], M371Default, RXChannel);
						break;
					}
					case 0x376:	// Model 376
					{
						printf("\nInitializing Red Rapids Model 376...\n");
						// Initialize hardware specific to the Model 376.
						Status += M376Initialize(p_HW[DEV0], M376Default, RXChannel);
						break;
					}
					case 0x377:	// Model 377
					{
						printf("\nInitializing Red Rapids Model 377...\n");
						// Initialize hardware specific to the Model 377.
						Status += M377Initialize(p_HW[DEV0], M377Default, RXChannel);
						break;
					}
					default:	// Illegal model number.
					{
						Status++;
						printf("\nERROR: Model number reported by the hardware is not recognized.\n");
						break;
					}
				}

			// Initialize the clocks if there were no prior errors.
			if (!Status)
			{
				Status = ClockInit(p_HW[DEV0]);
				// Report the clock status returned from the hardware.
				switch (ClockStatus(p_HW[DEV0], CMD_STAT1))
				{
					case 0: {printf("\tExternal sample clock active, converter clocks locked.\n"); break;}
					case 1: {printf("\tERROR: External sample clock active, converter clocks not locked.\n"); Status++; break;}
					case 2: {printf("\tSample clock locked to internal reference, converter clocks locked.\n"); break;}
					case 3: {printf("\tERROR: Sample clock locked to internal reference, converter clocks not locked.\n"); Status++; break;}
					case 4: {printf("\tSample clock locked to external reference, converter clocks locked.\n"); break;}
					case 5: {printf("\tERROR: Sample clock locked to external reference, converter clocks not locked.\n"); Status++;break;}
					default: {printf("\tERROR: Sample clock is not locked.\n"); Status++; break;}
				}
			}

			// Initialize the user defined structure attached to the hardware handle.
			for (chancnt = 1; chancnt <= User.RXChannelCount + User.TXChannelCount; chancnt++)
			{
				if (chancnt > User.RXChannelCount)
					// Initialize the structure members assigned to transmitter channels if there were no prior errors.
					if (!Status)
					{
						Status = UserInitialize(&User, chancnt, TXChannel[chancnt-1].DMABuffer, DataBuffer[chancnt-1]);
						if (Status) printf("\tERROR: Unable to allocate memory for the channel #%lu DataBuffer.\n", chancnt);
					}
					// Mark all of the buffers as a failed memory allocation if there were prior errors.
					else
						for (index = 0; index < TXChannel[chancnt-1].DMABuffer.PagesPerMark; index++)
							DataBuffer[chancnt-1][index] = 1;
				else
					// Initialize the structure members assigned to receiver channels if there were no prior errors.
					if (!Status)
					{
						Status = UserInitialize(&User, chancnt, RXChannel[chancnt-1].DMABuffer, DataBuffer[chancnt-1]);
						if (Status) printf("\tERROR: Unable to allocate memory for the channel #%lu DataBuffer.\n", chancnt);
					}
					// Mark all of the buffers as a failed memory allocation if there were prior errors.
					else
						for (index = 0; index < RXChannel[chancnt-1].DMABuffer.PagesPerMark; index++)
							DataBuffer[chancnt-1][index] = 1;
			}
				
			// Set hardware configuration options if there were no prior errors.
			if (!Status)
			{
				// Set the direction of the GPIO and USER connector pins.
				// Note: The GPIOConnect() and/or USERConnect() functions must be called to enable each bus.
				LoadPinDirection(p_HW[DEV0], PinDirectionDefault);
				// Set the interrupt mask bits for individual fault conditions.
				Status = ErrorMask(p_HW[DEV0], ErrorMaskDefault);
				// Set the interrupt mask bits for individual channel status events.
				for (chancnt = 1; chancnt <= User.RXChannelCount + User.TXChannelCount; chancnt++)
					Status += ChannelMask(p_HW[DEV0], chancnt, ChannelMaskDefault);
				// Set the global interrupt mask.
				Status += InterruptMask(p_HW[DEV0], GlobalMaskDefault);
				// Report any errors.
				if (Status) printf("\tERROR: Unable to set hardware configuration options.\n");
			}

			// Configure each channel with application specific settings.
			for (chancnt = 1; chancnt <= User.RXChannelCount + User.TXChannelCount; chancnt++)
				// Configure the channel if there were no prior errors.
				if (!Status)
				{
					if (chancnt <= User.RXChannelCount)
						// Load receiver channel configuration settings.
						Status = LoadChannel(p_HW[DEV0], chancnt, RXChannel[chancnt-1]);
					else
						// Load transmitter channel configuration settings.
						Status = LoadChannel(p_HW[DEV0], chancnt, TXChannel[chancnt-1]);
					// A status value of 100 or greater indicates a failure to allocate the requested DMA buffer.
					if (Status < 100)
					{
						// Mark the successful allocation of a DMA buffer for this channel.
						DMABuffer[chancnt-1] = 0;
						// Transmitter channels must be preloaded with data before the initial DMA transfer is requested.
						if (chancnt > User.RXChannelCount)
						{
							// Load DAC samples into the TX data buffer, a sine wave is used here for demonstration purposes (Frequency = Fs / FsDivisor).
							// The TX data buffer will refresh the DMA buffer as data is consumed, but the contents remain static in this example.
							if (DataLoad(User.p_DataBuffer[chancnt-1], TXChannel[chancnt-1], FsDivisor, 16))
								printf("\tINFO: Discontinuities will be present in the channel #%lu TX waveform.\n", chancnt);
							// Preoload the DMA buffer with samples stored in the TX data buffer.
							for(pagecnt=0; pagecnt < TXChannel[chancnt-1].DMABuffer.PageCount; pagecnt++)
								memcpy(p_HW[DEV0]->DMA.VirtDMAAdr[chancnt-1][pagecnt], User.p_DataBuffer[chancnt-1][pagecnt%TXChannel[chancnt-1].DMABuffer.PagesPerMark], p_HW[DEV0]->DMA.BurstSizeB[chancnt-1]*p_HW[DEV0]->DMA.BurstCount[chancnt-1]);
						}
					}
					else
					{
						// Mark the failed allocation of a DMA buffer for this channel.
						DMABuffer[chancnt-1] = 1;
						printf("\tERROR: Unable to allocate memory for the channel #%lu DMABuffer.\n", chancnt);
					}
				}
				// Mark the DMA buffer as a failed memory allocation if there were prior errors.
				else
					DMABuffer[chancnt-1] = 1;

			// Perform a limited memory test if the product includes attached QDR II+ SRAM and there were no prior errors.
			if (!Status && MemSize(p_HW[DEV0]))
			{
				printf("\nStarting QDR II+ SRAM test...\n");
				printf("\tNumber of SRAM-A memory errors: %lu\n", MemTest(p_HW[DEV0],1));
				printf("\tNumber of SRAM-B memory errors: %lu\n", MemTest(p_HW[DEV0],2));
			}

			///////////////////////////////////////////////  Process Channels  ///////////////////////////////////////////////

			// Start channel processing if there were no prior errors and the receiver to transmitter loopback is not selected.
			if (!Status)
			{
				// Measure the maximum achievable PCI throughput by initiating continuous DMA transactions on all channels.
				// Note: This is not required to run an application, but it provides valuable system performance information.
					printf("\nMeasuring the PCI performance...\n");
					// Collect PCI performance data with maximum DMA transfers on all of the channels operating simultaneously.
					PciBenchmark(p_HW[DEV0]);
					// Retrieve the measured performance.
					PciPerformance(p_HW[DEV0], Performance);
					// Report the measured instantaneous PCI throughput.
					if (User.RXChannelCount) printf("\tMeasured RX PCI performance: %.3f Gbytes/sec\n", Performance[0]/1e9);
					if (User.TXChannelCount) printf("\tMeasured TX PCI performance: %.3f Gbytes/sec\n", Performance[1]/1e9);
					// Report the measured maximum PCI latency recorded.
					if (User.RXChannelCount) printf("\tMeasured RX PCI latency: %.3f usec\n", Performance[2]*4e-3);
					if (User.TXChannelCount) printf("\tMeasured TX PCI latency: %.3f nsec\n", Performance[3]*4e-3);
					// Clear the PCI latency performance measurement.
					ClearLatency(p_HW[DEV0]);

				// Start processing data on all available channels simultaneously.
				printf("\nStarting channels...\n");
				// Enable each channel to start processing when the specified start event occurs.
				for (chancnt = 1; chancnt <= User.RXChannelCount + User.TXChannelCount; chancnt++)
				{
					// Clear the channel status register.
					Status = ClearChannelStatus(p_HW[DEV0], chancnt);
					// Start the channel if there were no configuration errors.
					FlowControl(p_HW[DEV0], chancnt, 1);
					printf("\tChannel #%lu started.\n", chancnt);
				}

				// Monitor the status of all channels with a polling loop to determine when they are ready for a start trigger.
				do
				{
					Status = 0;
					// Pause before the next status query to minimize PCI bus traffic.
					MicrosecondTimer(100);
					// Check the ready status of each channel (0 = Not Ready, 1 = Ready).
					for (chancnt = 1; chancnt <= User.RXChannelCount + User.TXChannelCount; chancnt++)
						Status += ReadyStatus(p_HW[DEV0], chancnt);
				}
				while (Status != User.RXChannelCount + User.TXChannelCount);

				// Display an update message that includes the interrupt status.
				if (GlobalMaskDefault)
					printf("\nWaiting for all channels to stop processing (interrupts enabled)...\n");
				else
					printf("\nWaiting for all channels to stop processing (interrupts disabled)...\n");
				// It is not safe to abort execution while DMA transactions are active.
				printf("\tWARNING: CTRL-C FROM THE KEYBOARD MAY CRASH THE COMPUTER!\n");

				// Issue a software trigger to begin processing data on all channels.
				// Note: Different start/stop triggers can be selected in the channel configuration.
				// Note: Begin monitoring channel status immediately following the trigger so that DMA data is not lost.
				SoftwareTrigger(p_HW[DEV0]);
				
				// Monitor the status of all channels with a polling loop and terminate when all channels stop processing.
				do
				{
					Status = 0;
					// Check the on/off status of each channel to determine when they have completed processing.
					for (chancnt = 1; chancnt <= User.RXChannelCount + User.TXChannelCount; chancnt++)
						Status += FlowStatus(p_HW[DEV0], chancnt);
					// Process all channels for any status changes if interrupts are not enabled.
					if (!GlobalMaskDefault)
						ProcessChannels(p_HW[DEV0]);
					// Pause before the next channel status query to minimize PCI bus traffic.
					// Note: The pause should be much less than the time interval between DMA markers.
					if (Status) MicrosecondTimer(100);
				}
				while (Status);

				//////////////////////////////////////////////  Report Status  //////////////////////////////////////////////

				printf("\nRetrieving temperature status...\n");
				// Report the values measured by the temperature monitor.
				Status = GetTemperature(p_HW[DEV0], &Temperature);
				if (!Status)
				{
					printf("\tINFO: FPGA die temperature is %.2f degrees Celsius.\n", Temperature.Sensor[0]);
					printf("\tINFO: ADC proximity temperature is %.2f degrees Celsius.\n", Temperature.Sensor[1]);
					printf("\tINFO: QDR proximity temperature is %.2f degrees Celsius.\n", Temperature.Sensor[2]);
					printf("\tINFO: Shielded proximity temperature is %.2f degrees Celsius.\n", Temperature.Sensor[3]);
					printf("\tINFO: Card center proximity temperature is %.2f degrees Celsius.\n", Temperature.Sensor[4]);
				}
				else
					printf("\tERROR: Temperature monitor returned an error.\n");

				printf("\nRetrieving power status...\n");
				// Report the values measured by the power monitor.
				printf("\tINFO: Total power dissipation is %.2f Watts.\n", GetPower(p_HW[DEV0], &Power));
				printf("\tINFO: Supply #1 summary: %.2f Volts, %.2f Amps, %.2f Watts\n", Power.Supply1[0], Power.Supply1[1], Power.Supply1[2]);
				printf("\tINFO: Supply #2 summary: %.2f Volts, %.2f Amps, %.2f Watts\n", Power.Supply2[0], Power.Supply2[1], Power.Supply2[2]);
				// The third power supply is not present on native PCI Express products, do not report if the measured current is zero.
				if (Power.Supply3[1]) printf("\tINFO: Supply #3 summary: %.2f Volts, %.2f Amps, %.2f Watts\n", Power.Supply3[0], Power.Supply3[1], Power.Supply3[2]);

				printf("\nReviewing channel status...\n");
				// Report any faults that were detected during execution.
				Status = GetErrorStatus(p_HW[DEV0], &ErrorStatus);
				// Print each fault detected.
				if (ErrorStatus.Bar[0]) printf("\tERROR: Write to an illegal BAR0 address detected.\n");
				if (ErrorStatus.Bar[1]) printf("\tERROR: Read from an illegal BAR0 address detected.\n");
				if (ErrorStatus.Bar[2]) printf("\tERROR: Write to an illegal BAR2 address detected.\n");
				if (ErrorStatus.Bar[3]) printf("\tERROR: Read from an illegal BAR2 address detected.\n");
				if (ErrorStatus.Clock[0]) printf("\tERROR: 200 MHz reference out of lock.\n");
				if (ErrorStatus.Clock[1]) printf("\tERROR: Flash programmer clock out of lock.\n");
				if (ErrorStatus.Clock[2]) printf("\tERROR: QDR-A SRAM clock out of lock.\n");
				if (ErrorStatus.Clock[3]) printf("\tERROR: QDR-B SRAM clock out of lock.\n");
				if (ErrorStatus.Clock[4] || ErrorStatus.Clock[5] || ErrorStatus.Clock[6] || ErrorStatus.Clock[7]) printf("\tERROR: RX clock out of lock.\n");

				// Report any channel errors that were detected during execution.
				for (chancnt = 1; chancnt <= User.RXChannelCount + User.TXChannelCount; chancnt++)
				{
					if (User.Error[chancnt-1][0]) printf("\tERROR: Channel #%lu converter (ADC/DAC) error detected.\n", chancnt);
					if (User.Error[chancnt-1][1]) printf("\tERROR: Channel #%lu FIFO overflow/underflow, check PCI performance.\n", chancnt);
					if (User.Error[chancnt-1][2]) printf("\tERROR: Channel #%lu DMA data was lost, adjust the DMA marker to correct.\n", chancnt);
				}

				// Report the interrupt response time if interrupts were enabled.
				if (GlobalMaskDefault) printf("\tINFO: The last interrupt was serviced in %.2f microseconds.\n", 0.004*InterruptResponse(p_HW[DEV0]));

				// Extract the contents of each RX buffer and save to a file.
				if (User.RXChannelCount)
				{
					printf("\nSaving captured data on all receiver channels...\n");
					for (chancnt = 1; chancnt <= User.RXChannelCount; chancnt++)
					{
							// Save the first page of each RX data buffer to a file.
							printf("\tWriting channel #%lu data buffer contents to file %s.\n", chancnt, datafile[chancnt-1]);
							Status = DataSave(User.p_DataBuffer[chancnt-1], RXChannel[chancnt-1], datafile[chancnt-1], 1, 16);
							// Save the first page of each DMA buffer to a file.
							printf("\tWriting channel #%lu DMA buffer contents to file %s.\n", chancnt, dmafile[chancnt-1]);
							Status = DataSave(p_HW[DEV0]->DMA.VirtDMAAdr[chancnt-1], RXChannel[chancnt-1], dmafile[chancnt-1], 1, 16);
					}
				}
				printf("\n...all operations completed.\n");
				
			}
			else
				printf("\n...terminating execution due to hardware configuration errors.\n");
			
			// Free the memory allocated to the DMA buffers.
			for (chancnt = 1; chancnt <= User.RXChannelCount + User.TXChannelCount; chancnt++)
				if (!DMABuffer[chancnt-1]) Status = DMAFree(p_HW[DEV0], chancnt);

			// Close the Red Rapids device on the bus.
			Status = BusClose(p_HW[DEV0]);
			
		}
		else
			printf("\n...terminating execution because device could not be opened.\n");
	}
	else
		printf("\n...terminating execution due to hardware handle intialization errors.\n");


	/////////////////////////////////////////////////  Free Allocated Memory /////////////////////////////////////////////////

	// Free the memory allocated to the RX data buffers.
	for (chancnt = 1; chancnt <= User.RXChannelCount; chancnt++)
		for (index = 0; index < RXChannel[chancnt-1].DMABuffer.PagesPerMark; index++)
			if (!DataBuffer[chancnt-1][index]) free(User.p_DataBuffer[chancnt-1][index]);

	// Free the memory allocated to the TX data buffers.
	for (chancnt = User.RXChannelCount + 1; chancnt <= User.RXChannelCount + User.TXChannelCount; chancnt++)
		for (index = 0; index < TXChannel[chancnt-1].DMABuffer.PagesPerMark; index++)
			if (!DataBuffer[chancnt-1][index]) free(User.p_DataBuffer[chancnt-1][index]);

	// Free the memory allocated to the hardware handles.
	for (devcnt = 0; devcnt < MAX_NUM_DEVICES; devcnt++)
		if (!Handle[devcnt]) free(p_HW[devcnt]);

return 0;
}
